﻿/**************************************************************
* OutSystems Charts Mobile
*
* Last Update:  2016-03-23 by bmm
**************************************************************/
var ChartsMobile = (function() {
    
    var chartdata;
    var isToUpdate = false;
    
    
    var getClosest = function (elem, selector) {
        var firstChar = selector.charAt(0);
        // Get closest match
        for ( ; elem && elem !== document; elem = elem.parentNode ) {
            // If selector is a class
            if ( firstChar === '.' ) {
                if ( elem.classList.contains( selector.substr(1) ) ) {
                    return elem;
                }
            }
            // If selector is an ID
            if ( firstChar === '#' ) {
                if ( elem.id === selector.substr(1) ) {
                    return elem;
                }
            } 
            // If selector is a data attribute
            if ( firstChar === '[' ) {
                if ( elem.hasAttribute( selector.substr(1, selector.length - 2) ) ) {
                    return elem;
                }
            }
            // If selector is a tag
            if ( elem.tagName.toLowerCase() === selector ) {
                return elem;
            }
        }
        return false;
    };
    
    // Initialize chart or update series data
    var initChartData = function (data, callback, isClickable, yPrefix, ySufix) {
        
        var startTime = new Date().getTime();
        
        chartdata = JSON.parse(data);
        isToUpdate = false;
        
        //Check if chart is rendered
        if (Highcharts.charts.length > 0) {

             for (var i = 0; i< Highcharts.charts.length; i++) {
                if (Highcharts.charts[i]) {
                    // If the chart is already rendered update only data in series
                    if (Highcharts.charts[i].container.id === chartdata.containerId) {
                         isToUpdate = true;
                         /*for (var j = 0; j< Highcharts.charts[i].series.length; j++) {
                            Highcharts.charts[i].series[j].setData(chartdata.series[j].data, false);
                         }*/
                    }
                }
             }
             
        }
        
        //If this is a new rendering chart
        if (!isToUpdate) {
                
            // If Clickable set callback function in all data points
            if (isClickable) {
                var onclick = function (e) {  callback(e, this); };
                
                for (var i = 0; i < chartdata.series.length; i++) {
            
                    if (!chartdata.series[i].point) {
                       chartdata.series[i].point = {}; 
                    }
            
                    if (!chartdata.series[i].point.events) {
                        chartdata.series[i].point.events = {};
                    }
            
                    chartdata.series[i].point.events.click = onclick;
                }
        
            }
            
            //Set y Labels prefix and sufix if they're set 
            if (yPrefix !== '' ||ySufix !== '') {
                if (!chartdata.yAxis) {
                   chartdata.yAxis = {};
                }
                if (!chartdata.yAxis.labels) {
                   chartdata.yAxis.labels = {};
                }
                
                chartdata.yAxis.labels.formatter = function () { return (yPrefix !== '' ? yPrefix + ' ' : '') + this.axis.defaultLabelFormatter.call(this) + ySufix };
            } 
        }
        
        chartdata.credits = {};
        chartdata.credits.enabled = false;
        
        return { chartdata : chartdata, isToUpdate : isToUpdate };
    }
    
    //Config specific for chart type
    var configForChartType = function() {
        
        var startTime = new Date().getTime();
        
        switch(chartdata.chart.type) {
            
            case 'bar':
            
                if (!chartdata.tooltip) {
                    chartdata.tooltip = {};
                }
                chartdata.tooltip.formatter = function () { return (chartdata.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }

                // Recreate Series
                for (var i = 0; i < chartdata.series.length; i++) {
                    var data = [];
                    var catIndex = 0;
                    for(var j = 0; j < chartdata.xAxis.categories.length; j++) {
                        
                        if (chartdata.series[i].data[catIndex] && chartdata.series[i].data[catIndex].name === chartdata.xAxis.categories[j]) {
                           
                            var toPush = chartdata.series[i].data[catIndex].color !== '' ? { y : chartdata.series[i].data[catIndex].y, color : chartdata.series[i].data[catIndex].color, xTooltip : chartdata.series[i].data[catIndex].custom } : { y : chart.series[i].data[catIndex].y, xTooltip : chart.series[i].data[catIndex].custom };
                            data.push(toPush);
                            catIndex++;
                        }
                        else {
                            data.push(null);
                        }
                    }
                    
                    chartdata.series[i].data = data;
                }
                
                chartdata.plotOptions.line = {};
                chartdata.plotOptions.column = {};
                chartdata.plotOptions.area = {};
                chartdata.plotOptions.pie = {};
                
            break;
            
            case 'pie':
            
                chartdata.tooltip.formatter = function () { return (this.point.custom || (this.point.name + ': <b>' + this.y + '</b>')) };

                delete chartdata.xAxis.categories;
                chartdata.plotOptions.line = {};
                chartdata.plotOptions.column = {};
                chartdata.plotOptions.area = {};
                chartdata.plotOptions.bar = {};
                
            break;
            
            case ('line') :
            case ('area') :
            
                if (!chartdata.tooltip) {
                    chartdata.tooltip = {};
                }

                if (chartdata.xAxis.type === 'linear') {
                    
                    for (var i = 0; i < chartdata.series.length; i++) {
                        var data = [];
                        var catIndex = 0;
                        for(var j = 0; j < chartdata.xAxis.categories.length; j++) {
                            
                            if (chartdata.series[i].data[catIndex] && chartdata.series[i].data[catIndex].name === chartdata.xAxis.categories[j]) {
                                var toPush = chartdata.series[i].data[catIndex].color !== '' ? { y : chartdata.series[i].data[catIndex].y, color : chartdata.series[i].data[catIndex].color, xTooltip : chartdata.series[i].data[catIndex].custom } : { y : chartdata.series[i].data[catIndex].y, xTooltip : chartdata.series[i].data[catIndex].custom};
                                data.push(toPush);
                                catIndex++;
                            }
                            else {
                                data.push(null);
                            }
                        }
                        
                        chartdata.series[i].data = data;
                    }
                    
                    chartdata.tooltip.formatter = function () { return this.point.xTooltip ? this.point.xTooltip : (chartdata.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
                }

                if (chartdata.xAxis.type === 'datetime') {
                    
                    var min, max, pointCount = 0;
                    
                    for (var i = 0; i < chartdata.series.length; i++) {
                        var data = [];
                        var count = 0;
                        for(var j = 0; j < chartdata.series[i].data.length; j++) {
                            var tempDate = chartdata.series[i].data[j].name.replace(' ','T');
                            var date = new Date(tempDate);
                            
                            if (!min || date < min) {
                                min = date;
                            }
                            if (!max || date > max) {
                                max = date;
                            }
                            if (chartdata.series[i].data[j].color !== '') {
                                data.push({ x: Date.UTC(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds()),  marker : { fillColor: chartdata.series[i].data[j].color }, y : chartdata.series[i].data[j].y, color : chartdata.series[i].data[j].color, xTooltip : chartdata.series[i].data[j].custom });
                            }
                            else {
                                data.push({ x : Date.UTC(date.geUTCtFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds()), y : chartdata.series[i].data[j].y, xTooltip : chartdata.series[i].data[j].custom });
                            }
                            count++;
                        }
                        if (pointCount < count) {
                            pointCount = count;
                        }
                        
                        chartdata.series[i].data = data;
                    }
                    
                    var dif = max.getTime() - min.getTime();
                    
                    chartdata.xAxis.tickInterval = Math.floor(dif / pointCount);  //24 * 3600 * 1000,
                    chartdata.xAxis.min = Date.UTC(min.getUTCFullYear(), min.getUTCMonth(), min.getUTCDate(), min.getUTCHours(), min.getUTCMinutes(), min.getUTCSeconds());
                    chartdata.xAxis.max = Date.UTC(max.getUTCFullYear(), max.getUTCMonth(), max.getUTCDate(), max.getUTCHours(), max.getUTCMinutes(), max.getUTCSeconds());
                    
                    chartdata.tooltip.formatter = function () { var date = new Date(this.x); return this.point.xTooltip ? this.point.xTooltip :  (chartdata.series.length>1 ? this.series.name + ', ' : '') + date.getFullYear() + '-' + ((date.getMonth()+1) < 10 ? '0' : '') + (date.getMonth()+1) + '-' + (date.getUTCDate() < 10 ? '0' : '') +date.getUTCDate() +':  <b>' + this.y +'</b>'; }

                    delete chartdata.xAxis.categories;
                }

                if (chartdata.xAxis.type === 'decimal') {
                    
                    for (var i = 0; i < chartdata.series.length; i++) {
                        var data = [];
                        for(var j = 0; j < chartdata.series[i].data.length; j++) {
                            data.push({ x : parseFloat(chartdata.series[i].data[j].name), y : chartdata.series[i].data[j].y, xTooltip : chartdata.series[i].data[j].custom});
                        }
                        chartdata.series[i].data = data;
                    }
                    
                     chartdata.tooltip.formatter = function () { return this.point.xTooltip ? this.point.xTooltip : (chartdata.series.length>1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
                     
                     delete chartdata.xAxis.categories;
                }

                chartdata.plotOptions.pie = {};
                chartdata.plotOptions.column = {};
                chartdata.plotOptions.area = {};
                chartdata.plotOptions.bar = {};

            break;
            
            case 'column':
                if (!chartdata.tooltip) {
                    chartdata.tooltip = {};
                }
                
                for (var i = 0; i < chartdata.series.length; i++) {
                    var data = [];
                    var catIndex = 0;
 
                    if(chartdata.series.length === 1) { 
    
                        for(var j = 0; j < chartdata.series[i].data.length; j++) {
                            if (chartdata.series[i].data[j].name === chartdata.xAxis.categories[catIndex]) {
                                var toPush = chartdata.series[i].data[j].color !== '' ? { y : chartdata.series[i].data[j].y, color : chartdata.series[i].data[j].color, xTooltip : chartdata.series[i].data[j].custom } : { y : chart.series[i].data[j].y, xTooltip : chart.series[i].data[j].custom };
                                data.push(toPush);
                        
                                catIndex++;
                        
                            }
                        }
                        
                    } else {
                        
                        for(var j = 0; j < chartdata.xAxis.categories.length; j++) {
                            
                            if (chartdata.series[i].data[catIndex] && chartdata.series[i].data[catIndex].name === chartdata.xAxis.categories[j]) {
                                
                                var toPush = chartdata.series[i].data[catIndex].color !== '' ? { y : chartdata.series[i].data[catIndex].y, color : chartdata.series[i].data[catIndex].color, xTooltip : chartdata.series[i].data[catIndex].custom } : { y : chart.series[i].data[catIndex].y, xTooltip : chart.series[i].data[catIndex].custom };
                                
                                data.push(toPush);
     
                                catIndex++;
     
                            }
                            else {
                                data.push(null);
                            }
                        }
                    
                    }
                    
                    chartdata.series[i].data = data;
                }
                    
                chartdata.tooltip.formatter = function () { return this.point.xTooltip ? this.point.xTooltip : (chartdata.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }

                chartdata.plotOptions.line = {};
                chartdata.plotOptions.pie = {};
                chartdata.plotOptions.area = {};
                chartdata.plotOptions.bar = {};
                
            break;
        }
        
    }
    
    // Merges two (or more) objects, giving the last one precedence
    var merge = function (target, source) {
        
        if ( typeof target !== 'object' ) {
            target = {};
        }
    
        for (var property in source) {  
            if (source.hasOwnProperty(property)) {  
                var sourceProperty = source[property];
                
                if (typeof sourceProperty === 'object') {
                    target[property] = merge( target[property], sourceProperty);
                    continue;
                }
                
                target[ property ] = sourceProperty;
            }
        }
    
        for (var k = 2, l = arguments.length; k < l; k++) {
            merge(target, arguments[k]);
        }
    
        var isArray;
        var length = 0;
        for(property in target) {
            if (parseInt(property) === 0) {
                isArray = true;
            }
            length++;
        }

        if (isArray) {
            target.length = length;
            target = Array.prototype.slice.call(target);
        }
    
        return target;
    }
    
    var parseJSON = function (json){
        
        if (json.length > 0 && json[0] != '{') {
            json = '{' + json + '}';
        }
        
        json = json.replace(/''/g,'""');
        
        var rx = /(?:"([^\\"]|\\.)*")|(?:'([^\\']|\\.)*')|(-?(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?)|(\w+)\s*:/g;

        // Makes the necessary adjustments to make the JSON deserialization have the same behavior as server-side (Java and .NET)
        // 1st group - strings inside double quotes => untouched
        // 2nd group - strings inside single quotes => replaced by double quotes
        // 3rd group - numbers => transformed to string so that JSON.parse does not convert them to javascript numbers and cause precision loss before we can use them
        // 4th group - properties without quotes => placed inside double quotes

        var cleanJSON = json.replace(rx, function (match, dqString, sqString, numbers, unquoted) {
            if (numbers) { return numbers}
            if (sqString) { return "\"" + match.substring(1, match.length - 1) + "\""; }
            if (unquoted) { return "\"" + unquoted + "\":"; }
            return match;
        });

        return (cleanJSON);
    }
    
    
    var mergeOptions = function(xAxisJSON, yAxisJSON, dataPointFormat, dataSeriesFormat, extensionJSON) {
        
        var startTime = new Date().getTime();
        
        var toMerge;
        var source = chartdata;
        
        // Merge xAxis
        if(xAxisJSON !== '') {          
            try {
                
                toMerge = JSON.parse("{ \"xAxis\" : " + parseJSON(xAxisJSON) + "}");
                source = merge(source, toMerge);
            }
            catch (e) {
                eval('toMerge = { xAxis :' + parseJSON(xAxisJSON) + '}');
                source = merge(source, toMerge);
            }
        }
         // Merge yAxis
        if(yAxisJSON !== '') {
            try {
                toMerge = JSON.parse("{ \"yAxis\" : " + parseJSON(yAxisJSON) + "}");
                source = merge(source, toMerge);
            }
            catch (e) {
                eval('toMerge = { yAxis :' + parseJSON(yAxisJSON) + '}');
                source = merge(source, toMerge);
            }
        }
        
        var dataPointFormat = JSON.parse(dataPointFormat);
        var dataSeriesFormat = JSON.parse(dataSeriesFormat);

        var chartType = source.chart.type;
        // Data Points
        for (var i = 0; i < dataPointFormat.length; i++) {
            if (chartType === 'column' || chartType === 'area' || chartType === 'bar') {
                var serie;
                var indexInCategory = -1;
                for (var j = 0; j < source.series.length; j++) {
                    if (source.series[j].name === dataPointFormat[i].DataPoint.DataSeriesName || source.series.length === 1) {
                        serie = source.series[j];
                        break;
                    }
                }
                
                if (serie) {
                    for(j = 0; j < source.xAxis.categories.length ;j++) {
                        if (source.xAxis.categories[j] === dataPointFormat[i].DataPoint.Label) {
                            indexInCategory = j;
                        }
                    }
                    
                    if (indexInCategory > -1) {
                        if (serie.data[indexInCategory] === dataPointFormat[i].DataPoint.Value || serie.data[indexInCategory].y === dataPointFormat[i].DataPoint.Value) {
                             if (chartType === 'column') {
                                var toMerge = serie.data[indexInCategory];
                                try {
                                    toMerge = merge(toMerge, JSON.parse(parseJSON(dataPointFormat[i].DataPointJSON)));
                                }
                                catch(e) {
                                     eval('toMerge = merge(toMerge, ' + parseJSON(dataPointFormat[i].DataPointJSON) + ');');
                                }
                            }
                            if (chartType === 'area'  || chartType === 'bar') {
                                var obj;
                                try {
                                    obj = merge ( {y : serie.data[indexInCategory]} , JSON.parse(parseJSON(dataPointFormat[i].DataPointJSON)));
                                    serie.data[indexInCategory] = obj;
                                }
                                catch(e) {
                                    eval('ob = merge({y : ' + serie.data[indexInCategory].toString() + '}, ' + parseJSON(dataPointFormat[i].DataPointJSON) + ');');
                                    serie.data[indexInCategory] = ob;
                                    
                                }
                            }
                        }
                    }
                }
            }
            
            if (chartType === 'line') {
                var serie;
                for (var j = 0; j < source.series.length; j++) {
                    if (source.series[j].name === dataPointFormat[i].DataPoint.DataSeriesName  || source.series.length === 1) {
                        serie = source.series[j];
                        break;
                    }
                }
                
                if (serie) {
                   for(var j = 0; j < serie.data.length; j++) {
                       if (source.xAxis.type === 'datetime') {
                           //Keep ==
                           if (serie.data[j].y === dataPointFormat[i].DataPoint.Value && serie.data[j].x == (new Date( dataPointFormat[i].DataPoint.Label)).getTime() ) {
                               var toMerge = serie.data[j];
                               try {
                                    toMerge = merge(toMerge, JSON.parse(parseJSON(dataPointFormat[i].DataPointJSON)));
                                }
                                catch(e) {
                                     eval('toMerge = merge(toMerge, ' + parseJSON(dataPointFormat[i].DataPointJSON) + ');');
                                }
                                break;
                           }
                       } 
                       else {
                           var indexInCategory = -1;
                           for(j = 0; j < source.xAxis.categories.length ;j++) {
                                if (source.xAxis.categories[j] === dataPointFormat[i].DataPoint.Label) {
                                    indexInCategory = j;
                                    break;
                                }
                            }
                            if (indexInCategory > -1) {
                                if (serie.data[indexInCategory].y === dataPointFormat[i].DataPoint.Value) {
                                   var toMerge = serie.data[indexInCategory];
                                   try {
                                        toMerge = merge(toMerge, JSON.parse(parseJSON(dataPointFormat[i].DataPointJSON)));
                                    }
                                    catch(e) {
                                         eval('toMerge = merge(toMerge, ' + parseJSON(dataPointFormat[i].DataPointJSON) + ');');
                                    }
                                    break;
                               }
                                
                            }
                           
                       }
                       
                   }
                    
                }
            }
            
            if (chartType === 'pie') {
                var serie;

                for (var j = 0; j < source.series.length; j++) {
                    if (source.series[j].name === dataPointFormat[i].DataPoint.DataSeriesName || source.series.length === 1) {
                        serie = source.series[j];
                        break;
                    }
                }
                
                if (serie) {
                     for (var j = 0; j < serie.data.length; j++) {
                        if (serie.data[j].y === dataPointFormat[i].DataPoint.Value) {
                            var toMerge = serie.data[j];
                            try {
                                toMerge = merge(toMerge, JSON.parse(parseJSON(dataPointFormat[i].DataPointJSON)));
                            }
                            catch(e) {
                                 eval('toMerge = merge(toMerge, ' + parseJSON(dataPointFormat[i].DataPointJSON) + ');');
                            }
                        }
                         
                     }
                   
                    
                }
            }
        }
        // Data Series
        for (var i = 0; i < dataSeriesFormat.length; i++) {
   
           var serie;

            for (var j = 0; j < source.series.length; j++) {
                if (source.series[j].name === dataSeriesFormat[i].DataSeriesName || source.series.length === 1) {
                    serie = source.series[j];
                    break;
                }
            }
            
            if (serie) {
                
                var toMerge = serie;
                try {
                    toMerge = merge(toMerge, JSON.parse(parseJSON(dataSeriesFormat[i].DataSeriesJSON)));
                }
                catch(e) {
                     eval('toMerge = merge(toMerge, ' + parseJSON(dataSeriesFormat[i].DataSeriesJSON) + ');');
                }
            }
                
        }
        //Extension JSON
        if(extensionJSON !== '') {
            
            try {
                toMerge = JSON.parse(parseJSON(extensionJSON));
                source = merge(source, toMerge);
            }
            catch (e) {
                eval(' toMerge =' + parseJSON(extensionJSON));
                source = merge(source, toMerge);
            }
        }
        
        return source;
    }
    
    //Ident object in string format
    var insertIdent = function (ident) {
        var str = '';
        for(var i = 0; i< ident*3; i++) {
            str += ' ';
        }
        
        return str;
    }
    
    // Convert complex object to string
    var convertToText = function (obj, ident) {

        //create an array that will later be joined into a string.
        var string = [];

        if (obj === undefined) {
            return String(obj);
        } else if (typeof(obj) === "object" && obj !== null && (obj.join === undefined)) {
            for (var prop in obj) {
                if (obj.hasOwnProperty(prop))
                string.push(prop + ": " + convertToText(obj[prop], ident+1));
            }
        return "{\r\n" + insertIdent(ident+1) + string.join(",\r\n"+insertIdent(ident+1)) + "\r\n" + insertIdent(ident)+ "}";

        } else if (typeof(obj) === "object"  && obj !== null && (obj.join !== undefined)) {
            for(prop in obj) {
                string.push(convertToText(obj[prop], ident+1));
            }
        return "[" + string.join(",") + "]";

        } else if (typeof(obj) == "function") {
            string.push(obj.toString())

        } else {
            string.push(JSON.stringify(obj))
        }

        return string.join(",");
    }
    
    // Convert complex object to string without identation
    var convertToTextNoIdent = function (obj) {

        //create an array that will later be joined into a string.
        var string = [];

        //is object
        //    Both arrays and objects seem to return "object"
        //    when typeof(obj) is applied to them. So instead
        //    I am checking to see if they have the property
        //    join, which normal objects don't have but
        //    arrays do.
        if (obj === undefined) {
            return String(obj);
        } else if (typeof(obj) === "object" && obj !== null && (obj.join === undefined)) {
            for (var prop in obj) {
                if (obj.hasOwnProperty(prop))
                string.push(prop + ": " + convertToText(obj[prop]));
            }
        return "{" + string.join(",") + "}";

        //is array
        } else if (typeof(obj) === "object"  && obj !== null && (obj.join !== undefined)) {
            for(prop in obj) {
                string.push(convertToText(obj[prop]));
            }
        return "[" + string.join(",") + "]";

        //is function
        } else if (typeof(obj) == "function") {
            string.push(obj.toString())

        //all other values can be done with JSON.stringify
        } else {
            string.push(JSON.stringify(obj))
        }

        return string.join(",");
    }
    
    // Transform Luminance in RGB
    var colorLuminance = function (hex, lum) {

        // validate hex string
        hex = String(hex).replace(/[^0-9a-f]/gi, '');
        if (hex.length < 6) {
            hex = hex[0]+hex[0]+hex[1]+hex[1]+hex[2]+hex[2];
        }
        lum = lum || 0;

        // convert to decimal and change luminosity
        var rgb = "#", c, i;
        for (i = 0; i < 3; i++) {
            c = parseInt(hex.substr(i*2,2), 16);
            c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
            rgb += ("00"+c).substr(c.length);
        }

        return rgb;
    }
    
    // Transform decimal in hexadecimal
    var toHex = function (n) {
         n = parseInt(n,10);
         if (isNaN(n)) return "00";
         n = Math.max(0,Math.min(n,255));
         return "0123456789ABCDEF".charAt((n-n%16)/16)
              + "0123456789ABCDEF".charAt(n%16);
    }

    // Transform RGB in HEX 
    var colorToHex = function (color) {
        if (color.substr(0, 1) === '#') {
            return color;
        }
        var digits = /(.*?)rgb\((\d+), (\d+), (\d+)\)/.exec(color);
        
        var R = parseInt(digits[2]);
        var G = parseInt(digits[3]);
        var B = parseInt(digits[4]);
        
        return '#' + toHex(R)+toHex(G)+toHex(B);
    }
    
    // Convert rgb color to Hsl
    var convertToHsl = function (color){
        
        if (color.substr(0, 1) === '#') {
            return color;
        }
        var digits = /(.*?)rgb\((\d+), (\d+), (\d+)\)/.exec(color);
        
        var R = parseInt(digits[2])/255;
        var G = parseInt(digits[3])/255;
        var B = parseInt(digits[4])/255;
     
        var maxColor = Math.max(R,G,B);
        var minColor = Math.min(R,G,B);
        //Calculate L:
        var L = (maxColor + minColor) / 2 ;
        var S = 0;
        var H = 0;
        if(maxColor != minColor){
            //Calculate S:
            if(L < 0.5){
                S = (maxColor - minColor) / (maxColor + minColor);
            }else{
                S = (maxColor - minColor) / (2.0 - maxColor - minColor);
            }
            //Calculate H:
            if(R == maxColor){
                H = (G-B) / (maxColor - minColor);
            }else if(G == maxColor){
                H = 2.0 + (B - R) / (maxColor - minColor);
            }else{
                H = 4.0 + (R - G) / (maxColor - minColor);
            }
        }
     
        L = L * 100;
        S = S * 100;
        H = H * 60;
        if(H<0){
            H += 360;
        }
        var result = [H, S, L];
        return result;
    }
    
    // Render Chart
    var renderChart = function () {
        
         var startTime = new Date().getTime();
         
        var isDebugging = false;
        var debugBlock = getClosest(document.getElementById(chartdata.chart.renderTo), '.chart-debug');
        
        if (debugBlock) {
            
            var pre = debugBlock.querySelector('pre');
            if (pre && pre.innerHTML === '') {
                pre.innerHTML = convertToText(chartdata, 0);
            }
            var textarea = debugBlock.querySelector('textarea');
            if (textarea && textarea.value === '') {
                textarea.value = convertToText(chartdata, 0);
            } else {
                isDebugging = true;
            }
        }

        chartdata.plotOptions.series.point = {};
        chartdata.plotOptions.series.point.events = {};

        var classList = document.body.classList;
        var isTouch = (classList.contains("phone") || classList.contains("tablet"));

        //Need to work in mobile IOS
        if (isTouch) {
            
            switch (chartdata.chart.type) {
                case 'pie':
                    chartdata.plotOptions.series.point.events.legendItemClick = function () { return !(event instanceof MouseEvent) };
                break;
                
                case 'column':
                    if (!chartdata.plotOptions.column.events){
                        chartdata.plotOptions.column.events = {};
                    }
                    
                    chartdata.plotOptions.column.events.legendItemClick = function () { return !(event instanceof MouseEvent) };
                break;
                
                case 'bar':
                    if (!chartdata.plotOptions.bar.events){
                        chartdata.plotOptions.bar.events = {};
                    }
                    
                    chartdata.plotOptions.bar.events.legendItemClick = function () { return !(event instanceof MouseEvent) };
                break;
                
                case 'area':
                    if (!chartdata.plotOptions.area.events){
                        chartdata.plotOptions.area.events = {};
                    }
                    
                    chartdata.plotOptions.area.events.legendItemClick = function () { return !(event instanceof MouseEvent) };
                break;
                
                case 'line':
                    if (!chartdata.plotOptions.line.events){
                        chartdata.plotOptions.line.events = {};
                    }
                    
                    chartdata.plotOptions.line.events.legendItemClick = function () { return !(event instanceof MouseEvent) };
                break;
            }
            
        }

        var style = window.getComputedStyle(document.querySelector('.chart-container-wrapper .background-primary-color'),"");
        var mainColor = style.getPropertyValue("background-color");

        var hsl = convertToHsl(mainColor);
        
        mainColor = colorToHex(style.getPropertyValue("background-color"));
        
        if (hsl[2] > 60) {
            mainColor = colorLuminance(mainColor, -0.1);
        }
        
        var step;
        var colors = [];

        if (chartdata.chart.type === 'pie') {
            step = parseFloat(1.45 / chartdata.series[0].data.length);
            for(var i = -0.4; i < 1.05; i+= step) {
                colors.push(colorLuminance(mainColor, i));
            }
        } else {
            step = parseFloat(0.8 / chartdata.series.length);
            for(var i = -0.4; i < 0.4; i+= step) {
                colors.push(colorLuminance(mainColor, i));
            }
        }
        
        if (!chartdata.chart.events) {
            chartdata.chart.events = {};
        }
        
        /*chartdata.chart.events.load = function () { 
        }*/
        
        
        var legendItems = document.querySelectorAll('.highcharts-legend-item rect');
        for (var i = 0; i<legendItems.length; i++) {
            legendItems[i].setAttribute('rx','15');
            legendItems[i].setAttribute('width','12');
        }
        
        var OS_ChartTheme = {};
        OS_ChartTheme.colors = colors;
        Highcharts.setOptions(OS_ChartTheme);
        
        var containerId = '';
        
        if (!isDebugging) {
            var chart = new Highcharts.Chart(chartdata);
            containerId = chart.container.id;
        }
        
        return containerId;
    }
    
    //Clean empty objects
    var clean = function (obj) {

    for (var prop in obj) {
        if (typeof(obj[prop]) === 'object' &&  Object.keys(obj[prop]).length === 0) {
             delete obj[prop];            
        }
        else {
            if (typeof(obj[prop]) === 'object') {
                obj[prop] = clean(obj[prop]);
                if (Object.keys(obj[prop]).length === 0) {
                     delete obj[prop];            
                }
            } else if (obj[prop] === undefined) {
                    delete obj[prop];
                }
        }
    }

    return obj;
}

    // Return the differences between two objects
    var filter = function (obj1, obj2) {
        var result = {};
        for(var key in obj1) {
            if(obj2[key] != obj1[key]) result[key] = obj2[key];
            if(typeof obj2[key] === Array && typeof obj1[key] === Array) 
                result[key] = arguments.callee(obj1[key], obj2[key]);
            if(typeof obj2[key] == 'object' && typeof obj1[key] == 'object') 
                result[key] = arguments.callee(obj1[key], obj2[key]);
        }
        for(key in obj2) {
            if(obj1[key] === undefined) result[key] = obj2[key];
        }
        
        return result;
    }
    
    // Retrieve the params for service studio
    var retrieveSS = function (widgetId) {

        var line = {
           chart: {
              plotShadow: false,
              type: "line",
              height: 300,
              animation: true,
              backgroundColor: "rgba(255, 255, 255, 0)"
           },
           title: {
              text: ""
           },
           tooltip: {
              formatter: function () { return this.point.xTooltip ? this.point.xTooltip : (chart.series.length>1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
           },
           plotOptions: {
              series: {
                 borderRadius: 1,
                 borderWidth: 0,
                 cursor: "",
                 point: {
                    events: {
                       
                    }
                 },
                 dataLabels: {
                    enabled: false,
                    format: ""
                 }
              },
              line: {
                 allowPointSelect: false,
                 cursor: "pointer",
                 dataLabels: {
                    enabled: false,
                    format: ""
                 },
                 showInLegend: true,
                 point: {
                    events: {
                       
                    }
                 },
                 marker: {
                    enabled: false
                 },
                 animation: true
              }
           },
           legend: {
              enabled: true,
              floating: false,
              itemMarginBottom: 10
           },
           xAxis: {
              labels: {
                 
              },
              title: {
                 text: ""
              }
           },
           yAxis: {
              title: {
                 text: ""
              },
              reversedStacks: true,
              stackLabels: {
                 enabled: false
              }
           },
           credits: {
              enabled: false
           }
        };
        var pie = {
               chart: {
                  plotShadow: false,
                  type: "pie",
                  height: 300,
                  animation: true,
                  backgroundColor: "rgba(255, 255, 255, 0)"
               },
               title: {
                  text: ""
               },
               tooltip: {
                  formatter: function () { return (this.point.custom || (this.point.name + ': <b>' + this.y + '</b>')) }
               },
               plotOptions: {
                  pie: {
                     allowPointSelect: true,
                     cursor: "pointer",
                     dataLabels: {
                        enabled: false,
                        format: ""
                     },
                     showInLegend: true,
                     point: {
                        events: {
                           
                        }
                     },
                     borderColor: 'rgba(255,255,255,0)',
                     animation: true
                  },
                  series: {
                     cursor: "",
                     borderRadius: 1,
                     borderWidth: 0,
                     point: {
                        events: {
                           
                        }
                     },
                     dataLabels: {
                        enabled: false,
                        format: ""
                     }
                  },
                  line: {
                     
                  },
                  column: {
                     
                  },
                  area: {
                     
                  },
                  bar: {
                     
                  }
               },
               legend: {
                  enabled: true,
                  floating: false,
                  itemMarginBottom: 10
               },
               xAxis: {
                  labels: {
                     
                  },
                  title: {
                     text: ""
                  }
               },
               yAxis: {
                  title: {
                     text: ""
                  },
                  reversedStacks: true,
                  stackLabels: {
                     enabled: false
                  }
               },
               credits: {
                  enabled: false
               }
            };
        var column = {
               chart: {
                  plotShadow: false,
                  type: "column",
                  height: 300,
                  animation: true,
                  backgroundColor: "rgba(255, 255, 255, 0)"
               },
               title: {
                  text: ""
               },
               tooltip: {
                  formatter: function () { return this.point.xTooltip ? this.point.xTooltip : (chart.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
               },
               plotOptions: {
                  series: {
                     cursor: "",
                     borderRadius: 1,
                     borderWidth: 0,
                     point: {
                        events: {
                           
                        }
                     },
                     dataLabels: {
                        enabled: false,
                        format: ""
                     }
                  },
                  column: {
                     allowPointSelect: false,
                     cursor: "pointer",
                     dataLabels: {
                        enabled: false,
                        format: ""
                     },
                     showInLegend: true,
                     point: {
                        events: {
                           
                        }
                     },
                     marker: {
                        enabled: false
                     },
                     animation: true
                  }
               },
               legend: {
                  enabled: true,
                  floating: false,
                  itemMarginBottom: 10
               },
               xAxis: {
                  labels: {
                  },
                  title: {
                     text: ""
                  }
               },
               yAxis: {
                  title: {
                     text: ""
                  },
                  reversedStacks: true,
                  stackLabels: {
                     enabled: false
                  }
               },
               credits: {
                  enabled: false
               }
            };
        var area = {
               chart: {
                  plotShadow: false,
                  type: "area",
                  height: 300,
                  animation: true,
                  backgroundColor: "rgba(255, 255, 255, 0)"
               },
               title: {
                  text: ""
               },
               tooltip: {
                  formatter: function () { return (chart.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
               },
               plotOptions: {
                  pie: {
                     
                  },
                  series: {
                     cursor: "",
                     borderRadius: 1,
                     borderWidth: 0,
                     point: {
                        events: {
                           
                        }
                     },
                     dataLabels: {
                        enabled: false,
                        format: ""
                     }
                  },
                  area: {
                     allowPointSelect: false,
                     cursor: "pointer",
                     dataLabels: {
                        enabled: false,
                        format: ""
                     },
                     showInLegend: true,
                     point: {
                        events: {
                           
                        }
                     },
                     marker: {
                        enabled: false
                     }
                  }
               },
               legend: {
                  align: "center",
                  enabled: true,
                  floating: false,
                  itemMarginBottom: 10
               },
               xAxis: {
                  labels: {
                     
                  },
                  title: {
                     text: ""
                  }
               },
               yAxis: {
                  title: {
                     text: ""
                  },
                  reversedStacks: true,
                  stackLabels: {
                     enabled: false
                  }
               },
               credits: {
                  enabled: false
               }
            };
        var bar = {
               chart: {
                  plotShadow: false,
                  type: "bar",
                  height: 300,
                  animation: true,
                  backgroundColor: "rgba(255, 255, 255, 0)"
               },
               title: {
                  text: ""
               },
               tooltip: {
                  formatter: function () { return (chart.series.length >1 ? this.series.name + ', ' : '') + this.x +':  <b>' + this.y +'</b>'; }
               },
               plotOptions: {
                  series: {
                     cursor: "",
                     borderRadius: 1,
                     borderWidth: 0,
                     point: {
                        events: {
                           
                        }
                     },
                     dataLabels: {
                        enabled: false,
                        format: ""
                     }
                  },
                  bar: {
                     allowPointSelect: false,
                     cursor: "pointer",
                     dataLabels: {
                        enabled: false,
                        format: ""
                     },
                     showInLegend: true,
                     point: {
                        events: {
                           
                        }
                     },
                     marker: {
                        enabled: false
                     }
                  }
               },
               legend: {
                  align: "center",
                  enabled: true,
                  floating: false,
                  itemMarginBottom: 10
               },
               xAxis: {
                  labels: {
                     
                  },
                  title: {
                     text: ""
                  }
               },
               yAxis: {
                  title: {
                     text: ""
                  },
                  reversedStacks: true,
                  stackLabels: {
                     enabled: false
                  }
               },
               credits: {
                  enabled: false
               }
            };
            
        var ss = {};
        var chartType;
        var newChart;

        eval('newChart = ' + document.getElementById(widgetId).querySelector('textarea').value + ';');

        var result;

        if (newChart.chart.type === 'column') {
            chartType = 'Column Chart';
            result = filter(column, newChart);
        } else
        if (newChart.chart.type === 'line') {
            chartType = 'Line Chart';
            result = filter(line, newChart);
            if (result && result.xAxis && result.xAxis.type === 'datetime') {
                delete result.xAxis.min;
                delete result.xAxis.max;
                delete result.xAxis.tickInterval;
            }
        } else 
        if (newChart.chart.type === 'pie') {
            result = filter(pie, newChart);
            if (result.plotOptions && result.plotOptions.series &&  result.plotOptions.series.innerSize === '50%') {
                chartType = 'Donut Chart';
                delete result.plotOptions.series.innerSize;
            } else {
                chartType = 'Pie Chart';
            }
        } else
        if (newChart.chart.type === 'area') {
            chartType = 'Area Chart';
            result = filter(area, newChart);
        } else 
        if (newChart.chart.type === 'bar') {
            chartType = 'Bar Chart';
            result = filter(bar, newChart);
        } else {
             chartType = 'Column Chart';
             result = filter(column, newChart);
        }

        if (result.chart && result.chart.renderTo) {
            delete result.chart.renderTo;
        }

        // Height
        if (result.chart.height) {
            ss.height = result.chart.height;
            delete result.chart.height;
        }

        // Stacking
        if (result.plotOptions && result.plotOptions.series && result.plotOptions.series.stacking === 'percent') {
            ss.stackingType = 'Entities.StackingType.Stacked100Percent';
            delete result.plotOptions.series.stacking;
            delete result.yAxis.reversedStacks;
        }

        if (result.plotOptions && result.plotOptions.series && result.plotOptions.series.stacking === 'normal') {
            ss.stackingType = 'Entities.StackingType.Stacked';
            delete result.plotOptions.series.stacking;
            delete result.yAxis.reversedStacks;
        }

        // Legend
        if (result.legend && result.legend.verticalAlign === "top" && result.legend.align === "right" && result.legend.layout === "vertical" && result.legend.floating === true) {
            ss.legend = 'Entities.LegendPosition.Inside';
            delete result.legend.verticalAlign;
            delete result.legend.align;
            delete result.legend.layout;
            delete result.legend.floating;
            delete result.legend.enabled;
        }

        if (result.legend && result.legend.verticalAlign === "middle" && result.legend.align === "right" && result.legend.layout === "vertical") {
            ss.legend = 'Entities.LegendPosition.Right';
            delete result.legend.verticalAlign;
            delete result.legend.align;
            delete result.legend.layout;
            delete result.legend.enabled;
        }

        if (result.legend && result.legend.verticalAlign === "middle" && result.legend.align === "left" && result.legend.layout === "vertical") {
            ss.legend = 'Entities.LegendPosition.Left';
            delete result.legend.verticalAlign;
            delete result.legend.align;
            delete result.legend.layout;
            delete result.legend.enabled;
        }

        if (result.legend && result.legend.align === "center") {
            ss.legend = 'Entities.LegendPosition.Bottom';
            delete result.legend.align;
            delete result.legend.enabled;
        }

        if (result.legend && result.legend.verticalAlign === "top") {
            ss.legend = 'Entities.LegendPosition.Top';
             delete result.legend.verticalAlign;
             delete result.legend.enabled;
        }

        if (result.legend && result.legend.enabled === false) {
            ss.legend = 'Entities.LegendPosition.Hidden';
            delete result.legend.enabled;
        }


        if (result.legend && result.legend.enabled === false) {
            ss.legend = 'Entities.LegendPosition.Hidden';
            delete result.legend.enabled;
        }

        //ChartFormat

        ss.ChartFormat = {};

        if (result.plotOptions && result.plotOptions.series && result.plotOptions.series.dataLabels) {
            ss.ChartFormat.ShowDataPointValues = result.plotOptions.series.dataLabels.enabled;
            delete result.plotOptions.series.dataLabels.enabled;
        }

        if ((result.plotOptions && result.plotOptions[newChart.chart.type] && result.plotOptions[newChart.chart.type].dataLabels && result.plotOptions[newChart.chart.type].dataLabels.enabled)) {
            ss.ChartFormat.ShowDataPointValues = result.plotOptions[newChart.chart.type].dataLabels.enabled;
            delete result.plotOptions[newChart.chart.type].dataLabels.enabled;
        }

        if (ss.stackingType !== '' && result.yAxis && result.yAxis.stackLabels && result.yAxis.stackLabels.enabled) {
            ss.ChartFormat.ShowDataPointValues = result.yAxis.stackLabels.enabled;
            delete result.yAxis.stackLabels.enabled;
        }


        if (result.chart !== {} && result.chart.animation!== undefined) {
            ss.ChartFormat.UseAnimation = result.chart.animation;
            delete result.chart.animation;
            delete result.plotOptions[newChart.chart.type].animation;
        } else {
            ss.ChartFormat.UseAnimation = true;
        }

        //xAxis Format
        ss.xAxis = {};

        if (result.xAxis && result.xAxis.title && result.xAxis.title.text){
            ss.xAxis.title = result.xAxis.title.text;
            delete result.xAxis.title;
        }

        if (result.xAxis && result.xAxis.labels && result.xAxis.labels.rotation) {
            ss.xAxis.labelsRotation = result.xAxis.labels.rotation;
            delete result.xAxis.labels.rotation;
        }

        if (result.xAxis && result.xAxis.min){
            ss.xAxis.minValue = result.xAxis.min;
            delete result.xAxis.min;
        }

        if (result.xAxis && result.xAxis.max){
            ss.xAxis.maxValue = result.xAxis.max;
            delete result.xAxis.max;
        }

        if (result.xAxis) {
             for (var child in result.xAxis) {
                if (result.xAxis[child] !== undefined) {
                    if (result.xAxis[child].categories) {
                        delete result.xAxis[child].categories;
                    } 
                    for (var c in result.xAxis[child]) {
                         if (result.xAxis[child][c] === undefined) {
                            delete result.xAxis[child][c];
                         }
                    }
                }
                else {
                    delete result.xAxis[child];
                }
            }
            if (result.xAxis["0"]) {
                result.xAxis = result.xAxis["0"];
            }
            delete result.xAxis.categories;
            delete result.xAxis.type;
        }

        // yAxis
        ss.yAxis = {};

        if (result.yAxis && result.yAxis.tickInterval) {
            ss.yAxis.GridLineStep = result.yAxis.tickInterval;
            delete result.yAxis.tickInterval;
        }

        if (result.yAxis && result.yAxis.max) {
            ss.yAxis.MaxValue = result.yAxis.max;
            delete result.yAxis.max;
        }

        if (result.yAxis && result.yAxis.min) {
            ss.yAxis.MinValue = result.yAxis.min;
            delete result.yAxis.min;
        }

        if (result.yAxis && result.yAxis.title && result.yAxis.title.text) {
            ss.yAxis.Title = result.yAxis.title.text;
            delete result.yAxis.title;
        }

        if (result.yAxis) {
             for (var child in result.yAxis) {
                if (result.yAxis[child] === undefined) {
                    delete result.yAxis[child];
                }
                else {
                    for (var c in result.yAxis[child]) {
                        if (result.yAxis[child][c] === undefined) {
                            delete result.yAxis[child][c];
                        }
                    }
                }
            }
        }

        ss.AdvanceFormat = {};
        ss.AdvanceFormat.XAxisJSON = {};
        ss.AdvanceFormat.YAxisJSON = {};
        ss.AdvanceFormat.HighchartsJSON = {};

        //XAxisJSON
        if (result.xAxis && result.xAxis.categories) {
            delete result.xAxis.categories;
        }
        if (result.xAxis && result.xAxis.type) {
            delete result.xAxis.type;
        }
        if (result.xAxis && result.xAxis.title  && Object.keys(result.xAxis.title).length === 0) {
            delete result.xAxis.title;
        }

        if (result.xAxis && result.xAxis.labels && Object.keys(result.xAxis.labels).length === 0) {
            delete result.xAxis.labels;
        }

        if (result.xAxis) {
            ss.AdvanceFormat.XAxisJSON = convertToTextNoIdent(result.xAxis).replaceAll('"','""');
            delete result.xAxis;
        }

        if (Object.keys(ss.AdvanceFormat.XAxisJSON).length === 0 || ss.AdvanceFormat.XAxisJSON === "{}") {
           ss.AdvanceFormat.XAxisJSON = ""; 
        }

        //YAxisJSON

        if (result.yAxis && result.yAxis.title  && Object.keys(result.yAxis.title).length === 0) {
            delete result.yAxis.title;
        }

        if (result.yAxis && result.yAxis.stackLabels && Object.keys(result.yAxis.stackLabels).length === 0) {
            delete result.yAxis.stackLabels;
        }

        if (result.yAxis) {
            ss.AdvanceFormat.YAxisJSON = convertToTextNoIdent(result.yAxis).replaceAll('"','""');;
            delete result.yAxis;
        }

        if (Object.keys(ss.AdvanceFormat.YAxisJSON).length === 0 || ss.AdvanceFormat.YAxisJSON === "{}") {
           ss.AdvanceFormat.YAxisJSON = ""; 
        }

        //DataSeriesFormats
        ss.DataSeriesFormats = [];

        if (result.series) {
            for(var i = 0; i <  result.series.length; i++) {
                if (result.series[i].data) {
                    delete result.series[i].data;
                }
                 if (result.series[i].point) {
                    delete result.series[i].point;
                }
                var dataSerieFormat = {};
                if (result.series[i].name) {
                    dataSerieFormat.DataSeriesName = result.series[i].name;
                    delete result.series[i].name;
                    for (var child in result.series[i]) {
                        if (result.series[i][child] === undefined) {
                            delete result.series[i][child];
                        }
                    }
                    dataSerieFormat.DataSeriesJSON =  convertToTextNoIdent(result.series[i]).replaceAll('"','""');;
                    if (Object.keys(result.series[i]).length > 1  && dataSerieFormat.DataSeriesJSON !== "{}") {
                        ss.DataSeriesFormats.push(dataSerieFormat);
                    }
                    
                }
            }
            delete result.series;
        }

        for (var child in result.chart) {
            if (result.chart[child] === undefined) {
                delete result.chart[child];
            }
        }
              
        for(var prop in result.plotOptions) {
            if (!result.plotOptions[prop] || Object.keys(result.plotOptions[prop]).length === 0) {
                delete result.plotOptions[prop];
            } else
            {
                for (var child in result.plotOptions[prop]) {
                    if (result.plotOptions[prop][child] === undefined) {
                        delete result.plotOptions[prop][child];
                    }
                }
            }
        }

        result.plotOptions = clean(result.plotOptions);

        for(var prop in result.tooltip) {
            if (!result.tooltip[prop] || Object.keys(result.tooltip[prop]).length === 0 || typeof(result.tooltip[prop][Object.keys(result.tooltip[prop])[0]]) === "function") {
                delete result.tooltip[prop];
            }
        }

        for(prop in result) {
            if (result[prop] === undefined || Object.keys(result[prop]).length === 0 || typeof(result[prop][Object.keys(result[prop])[0]]) === "function") {
                delete result[prop];
            }
        }

        ss.AdvanceFormat.HighchartsJSON = convertToTextNoIdent(result).replaceAll('"','""');;

        if (Object.keys(ss.AdvanceFormat.HighchartsJSON).length === 0 || ss.AdvanceFormat.HighchartsJSON === "{}") {
           ss.AdvanceFormat.HighchartsJSON = ""; 
        }

        var chart = {};
        chart.params = JSON.stringify(ss);
        chart.chartType = chartType;

        return chart;

        }

    return {
      
        init: function(data, callback, isClickable, yPrefix, ySufix) {    
            return initChartData(data, callback, isClickable, yPrefix, ySufix);     
        },
        configForChartType : function(data) {
            chartdata = data;
            configForChartType();
            return chartdata;
        },
        mergeOptions : function(data, xAxisJSON, yAxisJSON, dataPointFormat, dataSeriesFormat, extensionJSON) {
            chartdata = data;
            mergeOptions(xAxisJSON, yAxisJSON, dataPointFormat, dataSeriesFormat, extensionJSON);
            return chartdata;
        },
        renderChart : function(data) {
            chartdata = data;
            return renderChart();
        },
        retrieveSSParams : function(widgetId) {
            return retrieveSS(widgetId);
        }
    };

})();

String.prototype.replaceAll = function(search, replacement) {
    var target = this;
    return target.replace(new RegExp(search, 'g'), replacement);
};

