﻿/**
 * Vanilla Masker
 * @version 1.1.0
 * @author BankFacil
 * @license The MIT License (MIT)
 * @modified by OutSystems
 * @Last Update:  2017-03-20 by glc
**/
 
(function(root, factory) {
  if (typeof define === 'function' && define.amd) {
    define(factory);
  } else if (typeof exports === 'object') {
    module.exports = factory();
  } else {
    root.VMasker = factory();
  }
}(this, function() {
  var DIGIT = "9",
      ALPHA = "A",
      ALPHANUM = "*",
      BY_PASS_KEYS = [9, 16, 17, 18, 36, 37, 38, 39, 40, 91, 92, 93],
      changeOSVariable = function(element) {
        var event; // Custom event that will be created

        if(document.createEvent) {
          event = document.createEvent("HTMLEvents");
          event.initEvent("input", true, true);
        } else {
          event = document.createEventObject();
          event.eventType = "input";
        }

        event.eventName = "input";

        document.createEvent ? element.dispatchEvent(event) : element.fireEvent("on" + event.eventType, event);
      },
      isAllowedKeyCode = function(keyCode) {
        for (var i = 0, len = BY_PASS_KEYS.length; i < len; i++) {
          if (keyCode == BY_PASS_KEYS[i]) {
            return false;
          }
        }
        return true;
      },
      mergeMoneyOptions = function(opts) {
        opts = opts || {};
        opts = {
          precision: opts.hasOwnProperty("precision") ? opts.precision : 2,
          separator: opts.separator || "",
          delimiter: opts.delimiter || "",
          unit: opts.unit && (opts.unit.replace(/[\s]/g,'') + " ") || "",
          suffixUnit: opts.suffixUnit && (" " + opts.suffixUnit.replace(/[\s]/g,'')) || "",
          zeroCents: opts.zeroCents,
          lastOutput: opts.lastOutput
        };
        opts.moneyPrecision = opts.zeroCents ? 0 : opts.precision;
        return opts;
      },
      buildBasedRegex = function(output, useSpace) {
        
        if(typeof useSpace === "undefined") {
          useSpace = false;
        }

        if(typeof output === "undefined" || output === " " && useSpace === false) {
          output = "";
        }

        output = output.split("");

        for(i = 0; i < output.length; i++) {
          output[i] = "\\" + output[i];
        }

        return output.join("");

      },
      setCursorPosition = function(maskFunction, source, opts) {
        var position = opts.selectionStart,
            isDelimiter = false,
            isSeparator = false
        ;
        
        switch (maskFunction) {
          case "toMoney": 
            var number = source.value.toString().replace(/[\D]/g, ""),
                money = number.substr(0, number.length - opts.moneyPrecision),
                currencyValue = opts.suffixUnit ? source.value.substr(0, (source.value.length - opts.suffixUnit.length)) : source.value.substr(opts.unit.length, source.value.length),
                currencyLastOutput = opts.suffixUnit ? opts.lastOutput.substr(0, (opts.lastOutput.length - opts.suffixUnit.length)) : opts.lastOutput.substr(opts.unit.length, opts.lastOutput.length)
            ;
            
            // Cursor position after first mask
            if(opts.lastOutput === "" || (opts.selectionStart - 1) >= currencyValue.length && opts.suffixUnit || (opts.selectionStart - opts.unit.length - 1) >= currencyValue.length && opts.unit ) {
                position = opts.suffixUnit ? source.value.length - opts.suffixUnit.length : source.value.length;
            }
            
            // Invalid value
            if(source.value === opts.lastOutput && source.value.length > opts.lastOutput.length) {
                position = opts.selectionStart - (opts.originalValue.length - source.value.length);
            }

            // Inserting
            if(source.value.length > opts.lastOutput.length && opts.lastOutput !== "") {
                if((money.length - 1) % 3 === 0) {
                    isDelimiter = true;
                }
                
                if(isDelimiter) {
                    position = position + opts.delimiter.length;
                }
            }
            
            // Deleting
            if(source.value.length < opts.lastOutput.length && opts.lastOutput !== "" && opts.selectionStart <= currencyValue.length || source.value.length < opts.lastOutput.length && opts.lastOutput !== "" && (opts.selectionStart - opts.unit.length) <= currencyValue.length && opts.unit) {
                if(money.length % 3 === 0) {
                    isDelimiter = true;
                }
                
                if(isDelimiter) {
                    position = position - opts.delimiter.length;
                }
            }
            
            // Deleting DecimalDigits
            if(source.value.length === opts.lastOutput.length && parseFloat(currencyValue.replace(opts.delimiter,"").replace(opts.separator,".")) < parseFloat(currencyLastOutput.replace(".","").replace(",",".")) && source.value.length === (1 + opts.separator.length + opts.precision + (opts.unit || opts.suffixUnit).length)) {
                if(!opts.unit && source.value[0] === "0" || opts.unit && source.value[opts.unit.length] === "0" ) {
                    position = opts.selectionStart + 1;
                }
            }
            
          break;
          case "toCreditCard":
              
            // Inserting
            if(source.value.length > opts.lastOutput.length) {
                for(i = 0; i < opts.delimiter.length; i++) {
                    if(source.value[opts.selectionStart-1] === opts.delimiter[i]) {
                        isDelimiter = true;
                        break;
                    }
                }
                
                if(isDelimiter === true) {
                    position = opts.selectionStart + opts.delimiter.length;
                }
                
            // Unvalid value
            } else if(source.value === opts.lastOutput) {
                for(i = 0; i < opts.delimiter.length; i++) {
                    if(opts.lastOutput[opts.selectionStart] === opts.delimiter[i]) {
                        isDelimiter = true;
                        break;
                    }
                }
                
                if(isDelimiter === true) {
                    position = opts.selectionStart;
                } else {
                    position = opts.selectionStart - (opts.originalValue.length - source.value.length);
                }
                
            // Deleting
            } else {
                if(opts.selectionStart % (4 + opts.delimiter.length) === 0 && opts.selectionStart !== 0) {
                    position = opts.selectionStart - opts.delimiter.length;
                }
            }
          break;

        }
        
        source.setSelectionRange(position, position);
      }
  ;

  var VanillaMasker = function(elements) {
    this.elements = elements;
  };

  VanillaMasker.prototype.unbindElementToMask = function(maskFunction) {
    var unMask = function(source, opts) {
      if(opts.unit && source.value.length > 0 && source.value.match(/[0-9]/g) || opts.suffixUnit && source.value.length > 0 && source.value.match(/[0-9]/g) || !opts.unit && !opts.suffixUnit && source.value.length >= 0) {
        source.value = VMasker[maskFunction](source.value, JSON.parse(JSON.stringify(opts)));
      }
      changeOSVariable(source);
    };
    for (var i = 0, len = this.elements.length; i < len; i++) {
      this.elements[i].lastOutput = "";
      this.elements[i].onkeyup = false;
      this.elements[i].onkeydown = false;
      unMask(this.elements[i], this.opts);
      
    }
  };

  VanillaMasker.prototype.bindElementToMask = function(maskFunction) {
    var that = this,
        onType = function(e) {
          e = e || window.event;
          var source = e.target || e.srcElement,
              el = document.querySelector('#' + source.id),
              selectionStart = source.selectionStart
          ;

          if (isAllowedKeyCode(e.keyCode)) {
            setTimeout(function() {
              that.opts.lastOutput = source.lastOutput;
              that.opts.selectionStart = source.selectionStart;
              that.opts.originalValue = source.value;
              
              source.value = VMasker[maskFunction](source.value, that.opts);
              changeOSVariable(source);
              setCursorPosition(maskFunction, source, JSON.parse(JSON.stringify(that.opts)));
              
              if(source.value === "") {
                  source.lastOutput = "";
              } else {
                  source.lastOutput = source.value;
              }
              
            }, 0);
          }
        }
    ;
    
    for (var i = 0, len = this.elements.length; i < len; i++) {
      this.elements[i].lastOutput = "";
      this.elements[i].onkeyup = onType;
      if (this.elements[i].value.length) {
        this.elements[i].value = VMasker[maskFunction](this.elements[i].value, this.opts);
      }
    }
  };

  VanillaMasker.prototype.maskMoney = function(opts) {
    this.opts = mergeMoneyOptions(opts);

    opts.unit ? unit = opts.unit + " " : unit = opts.unit;
    opts.suffixUnit ? suffixUnit = " " + opts.suffixUnit : suffixUnit = opts.suffixUnit;

    this.bindElementToMask("toMoney");
  };

  VanillaMasker.prototype.unMaskMoney = function(opts) {
    this.opts = mergeMoneyOptions(opts);
    this.unbindElementToMask("fromMoney");
  };
  
  VanillaMasker.prototype.maskCreditCard = function(opts) {
    this.opts = {};
    this.opts.delimiter = opts.delimiter;
    this.bindElementToMask("toCreditCard");
  };

  VanillaMasker.prototype.unMaskCreditCard = function(opts) {
    this.opts = opts;
    this.unbindElementToMask("fromCreditCard");
  };

  VanillaMasker.prototype.unMask = function() {
    this.unbindElementToMask();
  };

  var VMasker = function(el) {
    if (!el) {
      throw new Error("VanillaMasker: There is no element to bind.");
    }
    var elements = ("length" in el) ? (el.length ? el : []) : [el];
    return new VanillaMasker(elements);
  };

  VMasker.toMoney = function(value, opts) {
    var decimalPart = "";
    opts = mergeMoneyOptions(opts);
    if (opts.zeroCents) {
      opts.lastOutput = opts.lastOutput || "";
      var zeroMatcher = ("("+ opts.separator +"[0]{0,"+ opts.precision +"})"),
          zeroRegExp = new RegExp(zeroMatcher, "g"),
          digitsLength = value.toString().replace(/[\D]/g, "").length || 0,
          lastDigitLength = opts.lastOutput.toString().replace(/[\D]/g, "").length || 0
      ;
      value = value.toString().replace(zeroRegExp, "");
      if (digitsLength < lastDigitLength) {
        value = value.slice(0, value.length - 1);
      }
    }
    
    var number = value.toString().replace(/[\D]/g, ""),
        clearDelimiter = (opts.delimiter !== "") ? new RegExp("^(0|\\"+ opts.delimiter +")") : new RegExp("^(0)"),
        clearSeparator = (opts.separator !== "") ? new RegExp("(\\"+ opts.separator +")$") : false,
        money = number.substr(0, number.length - opts.moneyPrecision),
        masked = money.substr(0, money.length % 3),
        cents = new Array(opts.precision + 1).join("0")
    ;
    money = money.substr(money.length % 3, money.length);
    for (var i = 0, len = money.length; i < len; i++) {
      if (i % 3 === 0) {
        masked += opts.delimiter;
        opts.addDelimiter = true;
      }
      masked += money[i];
    }
    
    masked = masked.replace(clearDelimiter, "");
    masked = masked.length ? masked : "0";
    if (!opts.zeroCents) {
      var beginCents = number.length - opts.precision,
          centsValue = number.substr(beginCents, opts.precision),
          centsLength = centsValue.length,
          centsSliced = (opts.precision > centsLength) ? opts.precision : centsLength
      ;
      cents = (cents + centsValue).slice(-centsSliced);
    }
    var output = opts.unit + masked + opts.separator + cents + opts.suffixUnit;
    output = output.replace(clearSeparator, "");
    
    for(i = 0; i < opts.precision; i++) {
        decimalPart += "0";
    }
    
    if(output !== opts.unit + "0" + opts.separator + decimalPart && output !== "0" + opts.separator + decimalPart + opts.suffixUnit && output !== "0") {
      return output;
    } else {
      return "";
    }
  };

  VMasker.fromMoney = function(value, opts) { // Unmask money function
  
    var output = value,
        clearSeparator = new RegExp(buildBasedRegex(opts.separator, true)),
        clearDelimiter = new RegExp(buildBasedRegex(opts.delimiter, true),'g'),
        clearUnit = new RegExp(buildBasedRegex(opts.unit, true)),
        clearSuffixUnit = new RegExp(buildBasedRegex(opts.suffixUnit, true)),
        aux
    ;
    
    output = output.replace(clearUnit,"");
    
    output = output.replace(clearSuffixUnit,"");
    
    output = output.replace(clearDelimiter,"");
    
    output = opts.separator ? output.replace(clearSeparator,".") : output;
    
    // Remove space at the end when suffixUnit exists
    if(opts.suffixUnit && output.indexOf(" ") > -1) {
        output = output.substr(0, output.length - 1);
    }
    
    // Remove space at the begining when unit exists
    if(opts.unit && output.indexOf(" ") > -1) {
        output = output.substr(1, output.length);
    }
    
    // Return zero when empty
    if(output === "") {
        output = 0;
    }

    return output;

  };
  
  VMasker.toCreditCard = function(value, opts) {
    var output = value,
        clearDelimiter = new RegExp(buildBasedRegex(opts.delimiter, true),'g'),
        regex = new RegExp(/^\d{0,16}$/g),
        aux = "",
        onlyNumbers
    ;

    output = output.replace(clearDelimiter,"");
    
    if(!output.match(regex)) {
        
        onlyNumbers = output.replace(/\D/g, "");
        
        if(typeof opts.lastOutput === "undefined" || onlyNumbers === "") {
            opts.lastOutput = "";
            console.log('The masked value is not valid');
        }
        return opts.lastOutput;
        
    }

    for(i = 0; i < output.length; i++) {
        if(i !== 0 && i % 4 === 0) {
            aux += opts.delimiter;
        }
        aux += output[i];
    }
    
    output = aux;
    
    return output;
  };
  
  VMasker.fromCreditCard = function(value, opts) {
    var output = value,
        clearDelimiter = new RegExp(buildBasedRegex(opts.delimiter, true),'g')
    ;
    
    output = output.replace(clearDelimiter,"");
    
    return output;
  };

  VMasker.updateOSVariable = function(element) {
    changeOSVariable(element);
  };
  
  return VMasker;
}));